{panaedra/msroot/msutil/logic/top_i/ctop_class.i}
using panaedra.msroot.msutil.logic.*.
using panaedra.msroot.msutil.interfaces.*.

class panaedra.msroot.msutil.logic.sc_dbtrig: 

  /****************************** Source Header ********************************\

      Description: This class is called by db delete and write triggers.
                   
                   It's also used for temp-tables and datasets, on a UI client
                   or a db-less appserver.
                   Temp-table get negative iID values.
                   
                   Table records get positive iID values, determined by a sequence
                   that is dedicated to the table.
                   
                   cGUID* is generated at creation of the temp-table record, or (db-only) at 
                   creation of the db table record.
                   If the db table record is based on a temp-table, cGUID* is simply copied.

  \*****************************  Include Files ********************************/
  {panaedra/msroot/msutil/logic/tt/msdbtrig_tableinfo_tt.i &class=class &protected=protected &static=static}
  /******************************* $NoKeywords:  $ *****************************/

  define protected static variable iNewID       as int64 no-undo.
  define protected static variable bInitialized as log   no-undo.
  define protected static variable bDbLayer     as log   no-undo.


  method public static logical OnWriteOrCreate (hBuffIP# as handle):
    
    /* Use the overload handle,handle,handle from db triggers.
       This method overload is strictly for temp-tables. 
     */

    define variable hBuffDummy# as handle no-undo.

    return OnWriteOrCreate(hBuffIP#, hBuffDummy#, hBuffDummy#).

  end method. /* OnWriteOrCreate ; single buffer overload */


  method public static logical OnWriteOrCreate (hBuffIP# as handle, hBuffOldIP# as handle):
    
    return OnWriteOrCreate(hBuffIP#, hBuffIP#, hBuffOldIP#).
    
  end method. /* OnWriteOrCreate ; current + old buffer overload */


  method public static void SetKeyFields(hBuffInputIP# as handle):
    
    define variable cCased# as character no-undo.

    cCased# = GetCased(hBuffInputIP#).
    
    SetKeyFields(hBuffInputIP#, cCased#).
    
  end method. /* SetKeyFields : simple overload */


  method public static void SetKeyFields(hBuffInputIP# as handle, cCasedIP# as char):
    
    /*
    
    Only fill the key fields (iID), and dtmMod* + dtmCre* fields.
    No validation logic is called.
    Also see sc_buffercopy:CopyKey (which calls SetKeyFields itself).
    
    */

    define variable hField# as handle no-undo.
    
    if not hBuffInputIP#:available then
      undo, throw new c_panaedraexception(99981726, sc_lang:oEnglish, substitute(
      "Record of '&1' is unavailable. Cannot set key fields.", hBuffInputIP#:name)).
    
    /* Fields */
        
    hField# = hBuffInputIP#:buffer-field("cGUID" + cCasedIP#) no-error.
    
    if hField# <> ? then 
    do:
      if sc_data:HasValue(hField#:buffer-value) then. /* not "" or ? */
      else hField#:buffer-value = NewGUID().
    end. 
    
    hField# = hBuffInputIP#:buffer-field("iID" + cCasedIP#) no-error.
    
    if hField# <> ? then 
    do:
      if hField#:buffer-value = ? or hField#:buffer-value <= 0 then 
      do:
        if 
          (/* db tables always get a positive iID */
          if hBuffInputIP#:table-handle = ? then true
        /* temp-tables kan keep their negative iID, but ? or 0 will be filled with a new iID. */ 
        else (hField#:buffer-value = ? or hField#:buffer-value = 0))
          then hField#:buffer-value = NewID(hBuffInputIP#, cCasedIP#).
      end.
    end.
    

    hField# = hBuffInputIP#:buffer-field("dtmMod" + cCasedIP#) no-error.
    
    if hField# <> ? then 
    do:
      hField#:buffer-value = now.
    end.
    

    hField# = hBuffInputIP#:buffer-field("dtmCre" + cCasedIP#) no-error.
    
    if hField# <> ? then 
    do:
      if hField#:buffer-value = ? then hField#:buffer-value = now.
    end.

  end method. /* SetKeyFields : efficient overload when cCased is already known. */


  method public static logical OnWriteOrCreate (hBuffIP# as handle, hBuffNewIP# as handle, hBuffOldIP# as handle):

    /* Fill ID if empty (negative for temp-table). 
       Fill cGUID*  if empty. 
       Fill dtmCre* if empty. 
       Fill dtmMod* always. */
    
    define variable hField#     as handle    no-undo.
    define variable cCased#     as character no-undo.
    define variable hBuffInput# as handle    no-undo.
    
    define buffer b-ttTableInfo for ttTableInfo.
    
    do transaction /* codeQok#7103 */
      on error undo, throw
      on stop undo, retry
      on endkey undo, retry:
        
      if retry then 
      do:
        undo, throw new Progress.Lang.AppError(subst("Error on write of '&1'", cCased#), 99952301).
      end.  
      
      if hBuffNewIP# <> ? and hBuffNewIP#:avail then hBuffInput# = hBuffNewIP#.
      else hBuffInput# = hBuffIP#.
      
      /* Each Panaedra table also has a cased version of the table name.
         The cased name is the "logical" name, which is used throughout the trigger logic.
         Cased name and noncased name can be very different,
         
       */

      if hBuffIP#:table-handle <> ? then 
        GetTtInfo(buffer b-ttTableInfo, hBuffIP#).
      else 
        GetDbInfo(true,buffer b-ttTableInfo, hBuffIP#).


      cCased# = b-ttTableInfo.cCased.
      
      SetKeyFields(hBuffInput#, cCased#).
      
      if hBuffIP#:table-handle = ? /* real db table */
        then 
      do:
        if ExecuteDbLogic(buffer b-ttTableInfo:handle, hBuffInput#, hBuffOldIP#, "wri") then.
        else return false.
      end.
      
      if ExecuteLogic(buffer b-ttTableInfo:handle, hBuffInput#, hBuffOldIP#, "wri") then.
      else return false.
      
    end. /* transaction codeQok#7103 */
    
    return true.

  end method. /* OnWriteOrCreate */


  method public static logical OnDelete (hBuffIP# as handle):

    define variable cCased#      as character no-undo.
    
    define variable hQueryRel#   as handle    no-undo.
    define variable hBuffTable#  as handle    no-undo.
    define variable hBuffRel#    as handle    no-undo.
    
    define variable hQueryChild# as handle    no-undo.
    define variable hBuffChild#  as handle    no-undo.
    
    define variable cIDParent#   as character no-undo.
    define variable iIDParent#   as int64     no-undo.
    
    define buffer b-ttTableInfo for ttTableInfo.

    if hBuffIP#:table-handle <> ? then 
      GetTtInfo(buffer b-ttTableInfo, hBuffIP#).
    else 
      GetDbInfo(true,buffer b-ttTableInfo, hBuffIP#).

    cCased# = b-ttTableInfo.cCased.
    
    do transaction /* codeQok#7103 */
      on error undo, throw
      on stop undo, retry
      on endkey undo, retry:
        
      if retry then 
      do:
        undo, throw new Progress.Lang.AppError(subst("Error on delete of '&1'", cCased#), 123002).
      end.

      if hBuffIP#:table-handle = ? /* real db table */
        then 
      do:
        if ExecuteDbLogic(buffer b-ttTableInfo:handle, hBuffIP#,?,"del") then.
        else return false.
      end.
            
      if ExecuteLogic(buffer b-ttTableInfo:handle, hBuffIP#,?,"del") then.
      else return false.
      
      sc_dbtrigrelations:GetDbtrigRelations(buffer b-ttTableInfo:handle, output hBuffTable#, output hBuffRel#).
       
      if hBuffTable# <> ?
        then
      do on error undo, throw:
        
        create query hQueryRel#.
        
        hQueryRel#:set-buffers(hBuffTable#, hBuffRel#).
        hQueryRel#:query-prepare(subst("for each &1 where &1.cTable = &2, each &3 of &1", hBuffTable#:name, quoter(b-ttTableInfo.cNonCased), hBuffRel#:name)).
        hQueryRel#:query-open.
        hQueryRel#:get-first.
        
        do while not hQueryRel#:query-off-end on error undo, throw:
          
          if hBuffIP#:table-handle = ? then
            create buffer hBuffChild# for table string(hBuffRel#::cChildTable).
          else
            /* ShouldHave: Datarelation temp-table delete; For dataset support a dataset context is necessary. */
            hBuffChild# = ?.
          
          if hBuffChild# <> ? then 
          do:  
            
            create query hQueryChild#.
            
            assign
              cIDParent# = "iID" + b-ttTableInfo.cCased
              iIDParent# = hBuffIP#:buffer-field(cIDParent#):buffer-value.
              
            hQueryChild#:set-buffers(hBuffChild#).
            hQueryChild#:query-prepare(subst("for each &1 where &1.&2 = &3 exclusive-lock" /* codeQok#7103 */, hBuffRel#::cChildTable, cIDParent#, iIDParent#)). 
            hQueryChild#:query-open.
            hQueryChild#:get-first.
            
            do while not hQueryChild#:query-off-end:
              hBuffChild#:buffer-delete.
              hQueryChild#:get-next.
            end. /* Walk childs */
            
          end. /* Valid child buffer */
          
          hQueryRel#:get-next.
          
          finally:
            if hQueryChild# <> ? then delete object hQueryChild#.
            if hBuffChild# <> ? then delete object hBuffChild#.
          end finally.
          
        end. /* Walk relations */
        
        finally:
          if hQueryRel# <> ? then delete object hQueryRel#.
        end.
        
      end. /* Valid Data Relations */
       
    end. /* transaction */
    
    return true.
    
  end method. /* OnDelete */
  

  method public static character NewGUID ():
    
    return replace(guid(generate-uuid),"-","").
    
  end method. /* NewGUID */
  
  
  method public static int64 NewID ( hBuffIP# as handle, cCasedIP# as char):
    
    define variable iReturn# as int64 no-undo.
    define variable rUuid#   as raw   no-undo.  

    define buffer b-ttTableInfo for ttTableInfo.


    if not bInitialized then 
    do:
      bInitialized = true.
      if sc_data:HasValue(panaedra.msroot.msutil.logic.sc_boot:cBootSessionParameter) then.
      else panaedra.msroot.msutil.logic.sc_boot:cBootSessionParameter = sc_string_substitute:ExpandOsEnv(session:parameter).
      bDbLayer = lookup(panaedra.msroot.msutil.logic.sc_boot:SessionParameter("Layer"),"all,db,db_r,db_u") > 0.
      if bDbLayer = ? then bDbLayer = false.
      rUuid# = generate-uuid.
      /* We don't want the same negative number because of possible ambiguity outside the session, or in case of persistent data. It's also handy for debugging.   */
      /* The generate-uuid is just a safety precaution for sessions that get their first iNewID at exactly the same time (in ms).                                  */
      /* We take the most variable byte of the uuid regarding short (<1ms) intervals, and convert it to a signed short. Then we append it to the ms interval.      */
      /* The "abs" is for poor people who's bios battery has died. It prevents logical errors within the session (find first cq last).                             */
      /* iNewID is always negative, to indicate that it's a temporary key for use with temp-tables. On the database persistency layer, the buffer gets a real key. */
      iNewID = -1 * (100000 * abs(interval(now,datetime-tz(1,1,2010,0,0,0),"milliseconds"))) + (32767 + get-short(rUuid#,12)).
    end.
    
    if not bDbLayer or hBuffIP#:table-handle <> ? then 
    do:
      assign
        iNewID   = iNewID - 1
        iReturn# = iNewID.
      if hBuffIP#:table-handle <> ? then 
      do:
        GetTtInfo(buffer b-ttTableInfo, hBuffIP#).
        if b-ttTableInfo.cPersistencyMode = "xml-store" then 
        do:
          /* If the temp-table is defined as "store to xml file" (not to db), we return a positive iID */
          iReturn# = (iNewID * -1).
        end.
      end.
      return iReturn#.
    end.
    else 
    do transaction on error undo, throw: /* codeQok#7103 , because wrong use of layer is prevented on runtime */
      return dynamic-next-value("s_ID" + cCasedIP#, hBuffIP#:table /* sc_dbconnect:SetAliases() is used; table name is alias db name */).
    end.

  end method. /* NewID */ 
  
  
  method public static void SetBufferNameProps(
    
    /* 
    
      Set standard panaedra name-properties of a temp-table buffer (this method can be extended for db buffers as well).
      This method is *NOT* needed for a standard panaedra-style defined temp-table.
      This method can be used for simple "like" temp-tables to quickly set properties like CasedTableName.
      
      Example:
         
      sc_dbtrig:SetBufferNameProps(
        buffer b-tt_CCMPNY_selectie:handle, 
        "Li_CCMPNY_Selectie", /* Cased name                 */
        "_CCMPNY_selectie",   /* DB Table name, or '(none)' */
        "l_root",          /* Namespace root             */ codeQok#7116
        "li_TERM_UNDISCLOSED_",       /* Namespace groupdir         */
        "li").             /* Namespace key              */    
    
    */
    
    hBuffIP#           as handle,
    cCasedTableNameIP# as character,
    cTableAliasIP#     as character,
    cNsSubrootIP#      as character,
    cNsGroupdirIP#     as character,
    cNsKeyIP#          as character
    ):
      
    define variable hFirstField# as handle no-undo.
    
    hFirstField# = hBuffIP#:buffer-field(1).
    
    sc_prop:SetProp(hFirstField#, "CasedTableName", cCasedTableNameIP#).  
    sc_prop:SetProp(hFirstField#, "TableAlias",     cTableAliasIP#).  
    sc_prop:SetProp(hFirstField#, "NsSubroot",      cNsSubrootIP#).  
    sc_prop:SetProp(hFirstField#, "NsGroupdir",     cNsGroupdirIP#).  
    sc_prop:SetProp(hFirstField#, "NsKey",          cNsKeyIP#).  
  
  end method. /* SetBufferNameProps */
  
  
  method public static char GetCased(hBuffIP# as handle):

    define variable cCased# as character no-undo.
    
    define buffer b-ttTableInfo for ttTableInfo.
    
    /* N.B. buffer or temp-table handle has no help attribute. 
       We use the help string (or GetProp/SetProp) of the first field of the (temp-)table buffer. 
    */

    if hBuffIP#:table-handle <> ? then 
      GetTtInfo(buffer b-ttTableInfo, hBuffIP#).
    else 
      GetDbInfo(true,buffer b-ttTableInfo, hBuffIP#).

    cCased# = b-ttTableInfo.cCased.

    return cCased#.
    
  end method. /* GetCased */

  
  method public static logical IsPanaedraized(hBuffIP# as handle):

    define buffer b-ttTableInfo for ttTableInfo.
    
    /* N.B. buffer or temp-table handle has no help attribute. 
       We use the help string (or GetProp/SetProp) of the first field of the (temp-)table buffer. 
    */

    if hBuffIP#:table-handle <> ? then 
      GetTtInfo(buffer b-ttTableInfo, hBuffIP#).
    else 
      GetDbInfo(false /* Does not have to be Panaedra Compliant */, buffer b-ttTableInfo, hBuffIP#).
    
    return avail b-ttTableInfo and sc_data:HasValue(b-ttTableInfo.cCased).
    
  end method. /* IsPanaedraized */

  
  method public static character GetNonCased(hBuffIP# as handle):

    define variable cNonCased#   as character no-undo.
    define variable hFirstField# as handle    no-undo.
    define variable cHelpString# as character no-undo.
    
    /* N.B. buffer or temp-table handle has no help attribute. 
       We use the help string (or GetProp/SetProp) of the first field of the (temp-)table buffer. 
    */
    
    hFirstField# = hBuffIP#:buffer-field(1).
    cHelpString# = hFirstField#:help.
    
    if cHelpString# matches "*[private-data]*" and hFirstField#:private-data = ? then
      sc_prop:PropsFromString(hFirstField#, sc_string_csv:GetParam(cHelpString#,"private-data")).
    
    /* TableAlias is optional, but is automatically generated from Panaedra 2010.01+. 
       If set (on the first buffer field), it overrules the default flow. 
       It can be used with "like" temp-tables, or "&ttPrefix/&ttSuffix" 
       temp-tables as used in changesets. 
    */
    cNonCased# = sc_prop:GetProp(hFirstField#,"TableAlias").

    if sc_data:HasValue(cNonCased#) then.
    else 
    do:
      if hBuffIP#:table-handle <> ? then 
      do:
        if hBuffIP#:table-handle:name begins "tt" 
          then cNonCased# = substring(hBuffIP#:table-handle:name,3).
        else cNonCased# = hBuffIP#:table-handle:name.
      end.
      else  
        cNonCased# = hBuffIP#:name.
    
      if cNonCased# = "" or cNonCased# = ? or 
        index(cNonCased#, " ") > 0 then
      do:
        if (hBuffIP#:table <> ?) then
          cNonCased# = hBuffIP#:table.
        else if (hBuffIP#:table-handle <> ?) then 
          do:
            cNonCased# = hBuffIP#:table-handle:name.
            if cNonCased# begins "tt" then cNonCased# = substring(cNonCased#,3).
          end.
      end.
      
    end. /* Property "TableAlias" was not used */
    
    /* For tables with an abstraction from table to temp-table; 
       I.E. table "_CCMPNY_" could have "Pr_CCMPNY_/_CCMPNY_" as label. 
    */
    cNonCased# = entry(num-entries(cNonCased#,"/"),cNonCased#,"/").
    
    cNonCased# = lc(cNonCased#).
    
    return cNonCased#.
    
  end method. /* GetNonCased */

  
  method public static void GetDbInfo(bMustBePanaedraCompliantIP# as logical, buffer b-ttTableInfo for ttTableInfo, hBuffIP# as handle):
    
    define variable bEclipseFix# as logical no-undo.
    
    define variable hQuery#      as handle  no-undo.
    define variable hBuff#       as handle  no-undo.
      
    find first b-ttTableInfo where b-ttTableInfo.cDbTableName = hBuffIP#:table no-error.
    
    if not avail b-ttTableInfo then 
    do:
      create b-ttTableInfo.
      assign 
        b-ttTableInfo.cDbTableName = hBuffIP#:table.
      create buffer hBuff# for table /* assumes sc_dbconnect:SetAliases() is used; */ hBuffIP#:table + "._File".
      hBuff#:find-first(subst("where _File._File-name = &1 ", quoter(hBuffIP#:table)), no-lock) no-error.
      if not hBuff#:available then undo, throw new Progress.Lang.AppError(subst("Table '&1' not found", hBuffIP#:table),99950285).
      if bMustBePanaedraCompliantIP# and not num-entries(hBuff#::_Desc,"/") >= 3 then undo, throw new Progress.Lang.AppError(subst("Table '&1' Description invalid: '&2'", hBuffIP#:table, hBuff#::_Desc),99950286).
      assign
        b-ttTableInfo.cNonCased        = hBuffIP#:table
        b-ttTableInfo.cLabel           = hBuff#::_File-Label
        b-ttTableInfo.cDescription     = hBuff#::_Desc
        b-ttTableInfo.cNsPath          = if (num-entries(hBuff#::_Desc,"/") < 3) then "" else entry(1,b-ttTableInfo.cDescription,"/")
        b-ttTableInfo.cCased           = if (num-entries(hBuff#::_Desc,"/") < 3) then "" else entry(2,b-ttTableInfo.cDescription,"/")
        b-ttTableInfo.cNsKey           = if (num-entries(hBuff#::_Desc,"/") < 3) then "" else entry(3,b-ttTableInfo.cDescription,"/")
        b-ttTableInfo.cNsDatabaseGroup = (if num-entries(b-ttTableInfo.cDescription,"/") < 4 then "" else trim(entry(4,b-ttTableInfo.cDescription,"/")))
        b-ttTableInfo.iTableMetaID     = (if num-entries(b-ttTableInfo.cDescription,"/") < 5 then 0 else int64(entry(5,b-ttTableInfo.cDescription,"/")))
        b-ttTableInfo.cNsSubroot       = if (num-entries(hBuff#::_Desc,"/") < 3) then "" else entry(1,b-ttTableInfo.cNsPath,".")
        b-ttTableInfo.cNsGroupdir      = if (num-entries(hBuff#::_Desc,"/") < 3) then "" else entry(2,b-ttTableInfo.cNsPath,".")
        b-ttTableInfo.cPersistencyMode = "oe-database"
        .
    end.
    
    finally:
      if valid-handle(hBuff#) then delete object hBuff#.
    end finally.
        
  end method. /* GetDbInfo */
  

  method protected static void GetTtInfo(buffer b-ttTableInfo for ttTableInfo, hBuffIP# as handle):
    
    define variable hFirstField# as handle    no-undo.
    define variable cHelpString# as character no-undo.

    find first b-ttTableInfo where b-ttTableInfo.cTempTableName = hBuffIP#:table-handle:name no-error.
    
    if not avail b-ttTableInfo then 
    do:
      
      create b-ttTableInfo.
      assign 
        b-ttTableInfo.cTempTableName = hBuffIP#:table-handle:name.
        
      hFirstField# = hBuffIP#:buffer-field(1).
      cHelpString# = hFirstField#:help.
      
      /* Properties may be set in the definition (in help string) or at run-time (with SetProp) */
      if cHelpString# matches "*[private-data]*" and hFirstField#:private-data = ? then
        sc_prop:PropsFromString(hFirstField#, sc_string_csv:GetParam(cHelpString#,"private-data")).
      
      assign
        b-ttTableInfo.cCased           = sc_prop:GetProp(hFirstField#,"CasedTableName")
        b-ttTableInfo.cNonCased        = sc_prop:GetProp(hFirstField#,"TableAlias")
        b-ttTableInfo.cNsKey           = sc_prop:GetProp(hFirstField#,"NsKey")
        b-ttTableInfo.cNsSubroot       = sc_prop:GetProp(hFirstField#,"NsSubroot")
        b-ttTableInfo.cNsGroupdir      = sc_prop:GetProp(hFirstField#,"NsGroupdir")
        b-ttTableInfo.cPersistencyMode = sc_prop:GetProp(hFirstField#,"PersistencyMode")
        .
      
      if trim(b-ttTableInfo.cCased) = "" or b-ttTableInfo.cCased = ? then 
        undo, throw new Progress.Lang.AppError(subst("Temp-table buffer '&1' (first field) has an invalid CasedTableName property of: '&2'", hBuffIP#:name, b-ttTableInfo.cCased),99950288).
      
    end.
        
  end method. /* GetTtInfo */  
  

  method public static void DebugShowTtCacheLogic():
    
    define variable bEclipseFix# as logical no-undo.
    
    define buffer b-ttCacheLogicLocations for ttCacheLogicLocations.

    if sc_html:TempTableToHTML(
      buffer b-ttCacheLogicLocations:handle, 
      sc_path:cTempDir + "ttCacheLogicLocations.html") 
      then 
      sc_win32:ShellExecute(sc_path:cTempDir + "ttCacheLogicLocations.html").

  end method. /* DebugShowTtCacheLogic */  
  
  
  method protected static logical ExecuteDbLogic(
    hBuffTableInfoIP# as handle, 
    hBuffIP#          as handle, 
    hBuffOldIP#       as handle, 
    cTypeIP#          as char):
    
    define variable cNsKey#            as character                   no-undo.
    define variable cNsDatabaseGroup#  as character                   no-undo.
    define variable oTriggerLogicData# as c_triggerlogicdata          no-undo.
    define variable oDbTrigClass#      as Progress.Lang.Class         no-undo.
    define variable oParamList#        as Progress.Lang.ParameterList no-undo.
    define variable bDbTrigOkay#       as logical                     no-undo.
    
    define buffer b-ttDbLogicLocations for ttDbLogicLocations.
    define buffer b-ttTableInfo        for ttTableInfo.
    
    if hBuffTableInfoIP# <> ? and hBuffTableInfoIP#:available 
      then 
      find first b-ttTableInfo where rowid(b-ttTableInfo) = hBuffTableInfoIP#:rowid.
    
    else 
    do:
      if hBuffIP#:table-handle <> ? then 
        GetTtInfo(buffer b-ttTableInfo, hBuffIP#).
      else 
        GetDbInfo(true,buffer b-ttTableInfo, hBuffIP#).
    end.
    
    assign
      cNsKey#           = b-ttTableInfo.cNsKey
      cNsDatabaseGroup# = b-ttTableInfo.cNsDatabaseGroup.
        
    do for b-ttDbLogicLocations:
    
      find first 
        b-ttDbLogicLocations where 
        b-ttDbLogicLocations.cNamespace = cNsKey# and
        b-ttDbLogicLocations.cNsDatabaseGroup = cNsDatabaseGroup#
        no-error.
      
      if not avail b-ttDbLogicLocations then 
      do:
        
        /* Write and delete trigger logic */
        create b-ttDbLogicLocations.
        assign 
          b-ttDbLogicLocations.cNamespace       = cNsKey#
          b-ttDbLogicLocations.cNsDatabaseGroup = cNsDatabaseGroup#.
      
        file-info:file-name = 
          "panaedra/" 
          + (if b-ttTableInfo.cNsSubroot = "(none)" then "" else b-ttTableInfo.cNsSubroot + "/")
          + b-ttTableInfo.cNsGroupdir + "/db_u/" +
          subst("sc_&1_&2onwriteordelete_u.cls",cNsKey#, (if sc_data:HasValue(cNsDatabaseGroup#) then cNsDatabaseGroup# + "_" else "")).
        if file-info:full-pathname = ? then 
        do:
          file-info:file-name = 
            "panaedra/" 
            + (if b-ttTableInfo.cNsSubroot = "(none)" then "" else b-ttTableInfo.cNsSubroot + "/")
            + b-ttTableInfo.cNsGroupdir + "/db_u/" +
            subst("sc_&1_&2onwriteordelete_u.r",cNsKey#, (if sc_data:HasValue(cNsDatabaseGroup#) then cNsDatabaseGroup# + "_" else "")).
        end.
        if file-info:full-pathname <> ? then 
        do:
          b-ttDbLogicLocations.cClass = replace(file-info:file-name,"~\","/").
          entry(num-entries(b-ttDbLogicLocations.cClass,"."),b-ttDbLogicLocations.cClass,".") = "".
          b-ttDbLogicLocations.cClass = right-trim(b-ttDbLogicLocations.cClass,". ").
          b-ttDbLogicLocations.cClass = replace(b-ttDbLogicLocations.cClass,"/",".").
        end.
                
      end. /* If not avail ; initialization, searching propath only once */  
      
      if b-ttDbLogicLocations.cClass <> "<none>" then 
      do:

        if not valid-object(oTriggerLogicData#) then
          oTriggerLogicData# = new panaedra.msroot.msutil.logic.c_triggerlogicdata().
        
        if not valid-object(b-ttDbLogicLocations.oDbTrigClass) then 
        do:
          oDbTrigClass# = Progress.Lang.Class:GetClass(b-ttDbLogicLocations.cClass).
          b-ttDbLogicLocations.oDbTrigClass = oDbTrigClass#.
        end.
        else
          oDbTrigClass# = cast(b-ttDbLogicLocations.oDbTrigClass, Progress.Lang.Class).

        if not valid-object(b-ttDbLogicLocations.oParamList) then 
        do:
          oParamList# = new Progress.Lang.ParameterList(5).
          b-ttDbLogicLocations.oParamList = oParamList#.
        end.
        else
          oParamList# = cast(b-ttDbLogicLocations.oParamList, Progress.Lang.ParameterList).

        oParamList#:SetParameter(1,"character","input",cTypeIP#).
        oParamList#:SetParameter(2,"CLASS panaedra.msroot.msutil.logic.c_triggerlogicdata","input",oTriggerLogicData#).
        oParamList#:SetParameter(3,"handle","input",hBuffIP#).
        oParamList#:SetParameter(4,"handle","input",hBuffOldIP#).
        oParamList#:SetParameter(5,"logical","output",bDbTrigOkay#).
        
        oDbTrigClass#:Invoke(subst("DbTrig_&1", b-ttTableInfo.cCased), oParamList#).

        oTriggerLogicData#:_sys_SetBuffers(hBuffIP#, hBuffOldIP#).

        if bDbTrigOkay# then.
        else
        do:
          /* ShouldHave: Later we can collect all data, but for now (proof of concept fase) only the first error is thrown. */
          if oTriggerLogicData#:_sys_HasErrorText() then 
            undo, throw new c_panaedraexception(
              9992731,
              sc_lang:oEnglish,
              substitute("DB logic error &1: '&2'",
              oTriggerLogicData#:_sys_FirstErrorNum(), 
              TriggerLogicErrorToString(hBuffIP#, hBuffOldIP#, oTriggerLogicData#:_sys_FirstErrorText()))
              ).
          else 
            undo, throw new c_panaedraexception(
              9992732,
              sc_lang:oEnglish,
              subst("A DB logic error has occurred for '&1'.", hBuffIP#:name)
              ).
        end.
      end.

    end. /* strong scoping of tt */
        
    return true.
    
    finally:
      if valid-object(oTriggerLogicData#) then delete object oTriggerLogicData#.
    end finally.
  
  end method. /* ExecuteDbLogic */
  

  method protected static logical ExecuteLogic (hBuffTableInfoIP# as handle, hBuffIP# as handle, hBuffOldIP# as handle, cTypeIP# as char):

    define variable bValidateOkay#     as logical            no-undo.
    define variable iError#            as integer            no-undo.
    define variable cError#            as character          no-undo.
    define variable cTable#            as character          no-undo.
    
    define variable oTriggerLogic#     as i_triggerlogic     no-undo.
    define variable oTriggerLogicData# as c_triggerlogicdata no-undo.
    
    define buffer b-ttCacheLogicLocations for ttCacheLogicLocations.
    define buffer b-ttTableInfo           for ttTableInfo.
    
    if hBuffTableInfoIP# <> ? and hBuffTableInfoIP#:available 
      then 
      find first b-ttTableInfo where rowid(b-ttTableInfo) = hBuffTableInfoIP#:rowid.
    
    else 
    do:
     
      if hBuffIP#:table-handle <> ? then 
        GetTtInfo(buffer b-ttTableInfo, hBuffIP#).
      else 
        GetDbInfo(true,buffer b-ttTableInfo, hBuffIP#).
    end.
    
    cTable# = b-ttTableInfo.cNonCased.
    
    do for b-ttCacheLogicLocations:
    
      find first 
        b-ttCacheLogicLocations where 
        b-ttCacheLogicLocations.cTable = cTable# and
        b-ttCacheLogicLocations.cType = cTypeIP# 
        no-error.
      
      if not avail b-ttCacheLogicLocations then 
      do:
        
        /* Validation logic */
        create b-ttCacheLogicLocations.
        assign 
          b-ttCacheLogicLocations.cTable = cTable#
          b-ttCacheLogicLocations.cType  = cTypeIP#.
      
        file-info:file-name = 
          "panaedra/" 
          + (if b-ttTableInfo.cNsSubroot = "(none)" then "" else b-ttTableInfo.cNsSubroot + "/")
          + b-ttTableInfo.cNsGroupdir + "/logic_val/c_"
          + cTypeIP# + "log_" + subst("&1",cTable#) + ".cls".
        if file-info:full-pathname = ? then 
        do:
          file-info:file-name = 
            "panaedra/" 
            + (if b-ttTableInfo.cNsSubroot = "(none)" then "" else b-ttTableInfo.cNsSubroot + "/")
            + b-ttTableInfo.cNsGroupdir + "/logic_val/c_"
            + cTypeIP# + "log_" + subst("&1",cTable#) + ".r".
        end.
        if file-info:full-pathname <> ? then 
        do:
          b-ttCacheLogicLocations.cLogicVal = replace(file-info:file-name,"~\","/").
          entry(num-entries(b-ttCacheLogicLocations.cLogicVal,"."),b-ttCacheLogicLocations.cLogicVal,".") = "".
          b-ttCacheLogicLocations.cLogicVal = right-trim(b-ttCacheLogicLocations.cLogicVal,". ").
          b-ttCacheLogicLocations.cLogicVal = replace(b-ttCacheLogicLocations.cLogicVal,"/",".").
        end.
        
        /* Calculation logic */
        file-info:file-name = 
          "panaedra/" 
          + (if b-ttTableInfo.cNsSubroot = "(none)" then "" else b-ttTableInfo.cNsSubroot + "/")
          + b-ttTableInfo.cNsGroupdir + "/logic_calc/c_"
          + cTypeIP# + "calc_" + subst("&1",cTable#) + ".cls".
        if file-info:full-pathname = ? then 
        do:
          file-info:file-name = 
            "panaedra/" 
            + (if b-ttTableInfo.cNsSubroot = "(none)" then "" else b-ttTableInfo.cNsSubroot + "/")
            + b-ttTableInfo.cNsGroupdir + "/logic_calc/c_"
            + cTypeIP# + "calc_" + subst("&1",cTable#) + ".r".
        end.
        if file-info:full-pathname <> ? then 
        do:
          b-ttCacheLogicLocations.cLogicCalc = replace(file-info:file-name,"~\","/").
          entry(num-entries(b-ttCacheLogicLocations.cLogicCalc,"."),b-ttCacheLogicLocations.cLogicCalc,".") = "".
          b-ttCacheLogicLocations.cLogicCalc = right-trim(b-ttCacheLogicLocations.cLogicCalc,". ").
          b-ttCacheLogicLocations.cLogicCalc = replace(b-ttCacheLogicLocations.cLogicCalc,"/",".").
        end.
        
      end. /* If not avail ; initialization, searching propath only once */  
    
      if b-ttCacheLogicLocations.cLogicVal <> "<none>" then 
      do:
        if not valid-object(oTriggerLogicData#) then
          oTriggerLogicData# = new panaedra.msroot.msutil.logic.c_triggerlogicdata().
        if cTypeIP# = "wri" then 
        do:
          oTriggerLogic# = dynamic-new b-ttCacheLogicLocations.cLogicVal (hBuffIP#, hBuffOldIP#, oTriggerLogicData#, output bValidateOkay#).
        end.
        else /* "del" */ 
        do:
          oTriggerLogic# = dynamic-new b-ttCacheLogicLocations.cLogicVal (hBuffIP#, oTriggerLogicData#, output bValidateOkay#).
        end.

        oTriggerLogicData#:_sys_SetBuffers(hBuffIP#, hBuffOldIP#).
        delete object oTriggerLogic# no-error. /* Data is not deleted. */
        oTriggerLogic# = ?.
      
        if bValidateOkay# then.
        else 
        do:
          /* ShouldHave: Later we can collect all data, but for now (proof of concept fase) only the first error is shown. */
          if oTriggerLogicData#:_sys_HasErrorText() then DisplayTriggerLogicError(hBuffIP#, hBuffOldIP#, oTriggerLogicData#:_sys_FirstErrorText()).
          else DisplayTriggerLogicError(hBuffIP#, hBuffOldIP#, subst("A data logic validation error has occurred for '&1'.", hBuffIP#:name)).
          return false.
        end.
      end.

      if b-ttCacheLogicLocations.cLogicCalc <> "<none>" then 
      do:
        if not valid-object(oTriggerLogicData#) then
          oTriggerLogicData# = new panaedra.msroot.msutil.logic.c_triggerlogicdata().
        if cTypeIP# = "wri" then 
        do:
          oTriggerLogic# = dynamic-new b-ttCacheLogicLocations.cLogicCalc (hBuffIP#, hBuffOldIP#, oTriggerLogicData#, output bValidateOkay#).
        end.
        else /* "del" */ 
        do:
          oTriggerLogic# = dynamic-new b-ttCacheLogicLocations.cLogicCalc (hBuffIP#, oTriggerLogicData#, output bValidateOkay#).
        end.

        oTriggerLogicData#:_sys_SetBuffers(hBuffIP#, hBuffOldIP#).
        delete object oTriggerLogic# no-error. /* Data is not deleted. */
        oTriggerLogic# = ?.
      
        if bValidateOkay# then.
        else 
        do:
          /* ShouldHave: Later we can collect all data, but for now (proof of concept fase) only the first error is shown. */
          if oTriggerLogicData#:_sys_HasErrorText() then DisplayTriggerLogicError(hBuffIP#, hBuffOldIP#, oTriggerLogicData#:_sys_FirstErrorText()).
          else DisplayTriggerLogicError(hBuffIP#, hBuffOldIP#, subst("A data logic calculation error has occurred for '&1'.", hBuffIP#:name)).
          return false.
        end.
      end.
    
    end. /* strong scoping, strange tt index problem */
        
    return true.
    
  end method. /* ExecuteLogic */


  method public static void DisplayTriggerLogicError (hBuffIP# as handle, hBuffOldIP# as handle, cErrorIP# as char):
    
    define variable cMessage# as character no-undo.
    
    cMessage# = TriggerLogicErrorToString(hBuffIP#, hBuffOldIP#, cErrorIP#).
    
    sc_messagebox:Show(sc_messageboxtype:Simple, cMessage#). 
    
  end method. /* DisplayTriggerLogicError */
  
  
  method protected static character TriggerLogicErrorToString(
    hBuffIP# as handle, hBuffOldIP# as handle, cErrorIP# as char):

    define variable cFieldNames#  as character no-undo extent 9.
    define variable cFieldValues# as character no-undo extent 9.
    define variable iIndex#       as integer   no-undo.
    define variable iExtent#      as integer   no-undo.
    define variable iTell#        as integer   no-undo.
    
    define variable cFields#      as character no-undo.
    define variable cOldValue#    as character no-undo.
    
    define variable cReturn#      as character no-undo.
    
    iIndex# = r-index(cErrorIP#,"[",length(cErrorIP#)).
    
    if iIndex# > 0 then 
    do:
      cFields# =  trim(substring(cErrorIP#,iIndex# + 1),"]").
      cErrorIP# = substring(cErrorIP#,1,iIndex# - 1).
    end.
    
    if sc_data:HasValue(cFields#) then
    do:
      iExtent# = num-entries(cFields#).
      do iTell# = 1 to num-entries(cFields#):
        cFieldNames#[iTell#] = entry(iTell#,cFields#).
        cFieldValues#[iTell#] = trim(string(hBuffIP#:buffer-field(cFieldNames#[iTell#]):buffer-value)).
        if hBuffOldIP# <> ? then 
        do:
          cOldValue# = trim(string(hBuffOldIP#:buffer-field(cFieldNames#[iTell#]):buffer-value)).
          if sc_data:HasValue(cOldValue#) and cOldValue# <> cFieldValues#[iTell#]
            then cFieldValues#[iTell#] = cOldValue# + " -> " + cFieldValues#[iTell#].
        end. 
      end.
    end.  
    
    cReturn# = subst(cErrorIP#, 
      cFieldValues#[1], cFieldValues#[2], cFieldValues#[3], cFieldValues#[4], 
      cFieldValues#[5], cFieldValues#[6], cFieldValues#[7], cFieldValues#[8], 
      cFieldValues#[9]).
    
    return cReturn#.
    
  end method. /* TriggerLogicErrorToString */


  method public static void SetDtmExpire(
  
    /* Fill the dtmExpire[Cased] field */
    
    hBuffInputIP# as handle,
    dtmExpireIP#  as datetime-tz):
      
    define variable cCased#   as character no-undo.
    define variable hBuffDyn# as handle    no-undo.

    cCased# = GetCased(hBuffInputIP#).
    
    create buffer hBuffDyn# for table hBuffInputIP#.

    do transaction on error undo, throw: /* codeQok#7103 */
      
      hBuffDyn#:find-by-rowid(hBuffInputIP#:rowid, exclusive-lock). /* codeQok#7102 */
      
      hBuffDyn#:buffer-field("dtmExpire" + cCased#):buffer-value = dtmExpireIP#.
      
      hBuffDyn#:buffer-release.
    
    end.
    
    finally:
      
      if valid-handle(hBuffDyn#) then delete object hBuffDyn#.
      
    end finally.
  
  end method. /* SetDtmExpire */

  
  method public static void DeleteExpiredRecords(
  
    /* Delete all records of a table, 
       whose dtmExpire[Cased] field is before "now".
       Locked records will be skipped.
     */
    
    cTableNameIP# as character):
      
    define variable cCased#      as character no-undo.
    define variable hBuffDyn#    as handle    no-undo.
    define variable hBuffDynDel# as handle    no-undo.
    define variable hQuery#      as handle    no-undo.
    define variable hFldExpire#  as handle    no-undo.
    define variable iBatchMax#   as integer   no-undo init 50.
    define variable iTell#       as integer   no-undo.
    
    create buffer hBuffDyn#    for table cTableNameIP#.
    assign
      cCased#     = GetCased(hBuffDyn#)
      hFldExpire# = hBuffDyn#:buffer-field("dtmExpire" + cCased#).
    
    create buffer hBuffDynDel# for table hBuffDyn#.
    create query  hQuery#.
    
    hQuery#:set-buffers(hBuffDyn#).
    
    hQuery#:query-prepare(substitute("preselect " +
      "each &1 no-lock where &1.&2 <= datetime-tz(&3)", 
      hBuffDyn#:name, 
      hFldExpire#:name, 
      quoter(now))).
      
    hQuery#:query-open().
    
    do while hQuery#:get-next():
      
      iTell# = iTell# + 1.
      
      if (iTell# - 1) mod iBatchMax# = 0 then 
      b_trans:
      do
        transaction /* codeQok#7103 */
        on error undo, throw:
        do while (not hQuery#:query-off-end): 
          hBuffDynDel#:find-by-rowid(hBuffDyn#:rowid, exclusive-lock, no-wait). /* codeQok#7102 */
          if not hBuffDynDel#:locked then
          do:
            hBuffDynDel#:buffer-delete().
            OnDelete(hBuffDynDel#).
          end.
          if (iTell# mod iBatchMax# = 0) then leave b_trans.
          hQuery#:get-next().
          iTell# = iTell# + 1.
        end.
      end.
      
    end.
    
    finally:
      
      if valid-handle(hBuffDyn#)    then delete object hBuffDyn#.
      if valid-handle(hBuffDynDel#) then delete object hBuffDynDel#.
      if valid-handle(hQuery#)      then delete object hQuery#.
      
    end finally.
  
  end method. /* SetDtmExpire */
  
end class.

/* EOF */
